@@ -17,6 +17,10 @@
 package org.gradle.api.internal.collections;
 
 import org.gradle.api.Action;
+import org.gradle.api.internal.DefaultMutationGuard;
+import org.gradle.api.internal.MutationGuard;
+import org.gradle.api.internal.provider.ChangingValue;
+import org.gradle.api.internal.provider.CollectionProviderInternal;
 import org.gradle.api.internal.provider.ProviderInternal;
 
 import java.util.Collection;
@@ -27,6 +31,7 @@ import java.util.TreeSet;
 public class SortedSetElementSource<T> implements ElementSource<T> {
     private final TreeSet<T> values;
     private final PendingSource<T> pending = new DefaultPendingSource<T>();
+    private final MutationGuard mutationGuard = new DefaultMutationGuard();
 
     public SortedSetElementSource(Comparator<T> comparator) {
         this.values = new TreeSet<T>(comparator);
@@ -107,7 +112,16 @@ public class SortedSetElementSource<T> implements ElementSource<T> {
     }
 
     @Override
-    public boolean addPending(ProviderInternal<? extends T> provider) {
+    public boolean addPending(final ProviderInternal<? extends T> provider) {
+        if (provider instanceof ChangingValue) {
+            ((ChangingValue<T>)provider).onValueChange(new Action<T>() {
+                @Override
+                public void execute(T previousValue) {
+                    values.remove(previousValue);
+                    pending.addPending(provider);
+                }
+            });
+        }
         return pending.addPending(provider);
     }
 
@@ -116,6 +130,27 @@ public class SortedSetElementSource<T> implements ElementSource<T> {
         return pending.removePending(provider);
     }
 
+    @Override
+    public boolean addPendingCollection(final CollectionProviderInternal<T, ? extends Iterable<T>> provider) {
+        if (provider instanceof ChangingValue) {
+            ((ChangingValue<Iterable<T>>)provider).onValueChange(new Action<Iterable<T>>() {
+                @Override
+                public void execute(Iterable<T> previousValues) {
+                    for (T value : previousValues) {
+                        values.remove(value);
+                    }
+                    pending.addPendingCollection(provider);
+                }
+            });
+        }
+        return pending.addPendingCollection(provider);
+    }
+
+    @Override
+    public boolean removePendingCollection(CollectionProviderInternal<T, ? extends Iterable<T>> provider) {
+        return pending.removePendingCollection(provider);
+    }
+
     @Override
     public void onRealize(Action<T> action) {
         pending.onRealize(action);
@@ -125,4 +160,9 @@ public class SortedSetElementSource<T> implements ElementSource<T> {
     public void realizeExternal(ProviderInternal<? extends T> provider) {
         pending.realizeExternal(provider);
     }
+
+    @Override
+    public MutationGuard getMutationGuard() {
+        return mutationGuard;
+    }
 }
