@@ -16,14 +16,19 @@
 package org.gradle.api.internal.artifacts.dsl;
 
 import org.gradle.api.Action;
-import org.gradle.api.artifacts.ComponentModuleMetadata;
+import org.gradle.api.artifacts.ComponentModuleMetadataDetails;
 import org.gradle.api.artifacts.dsl.ComponentModuleMetadataHandler;
 import org.gradle.api.internal.artifacts.ComponentModuleMetadataProcessor;
+import org.gradle.api.internal.artifacts.ImmutableModuleIdentifierFactory;
 
 public class DefaultComponentModuleMetadataHandler implements ComponentModuleMetadataHandler, ComponentModuleMetadataProcessor {
-    private final ComponentModuleMetadataContainer moduleMetadataContainer = new ComponentModuleMetadataContainer();
+    private final ComponentModuleMetadataContainer moduleMetadataContainer;
 
-    public void module(Object moduleNotation, Action<? super ComponentModuleMetadata> rule) {
+    public DefaultComponentModuleMetadataHandler(ImmutableModuleIdentifierFactory moduleIdentifierFactory) {
+        this.moduleMetadataContainer = new ComponentModuleMetadataContainer(moduleIdentifierFactory);
+    }
+
+    public void module(Object moduleNotation, Action<? super ComponentModuleMetadataDetails> rule) {
         rule.execute(moduleMetadataContainer.module(moduleNotation));
     }
 
