@@ -18,24 +18,20 @@ package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.builder
 
 import com.google.common.base.Joiner;
 import com.google.common.collect.Lists;
-import com.google.common.collect.Sets;
 import org.gradle.api.Describable;
-import org.gradle.api.Transformer;
 import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.api.artifacts.component.ComponentSelector;
-import org.gradle.api.artifacts.component.ModuleComponentSelector;
+import org.gradle.api.artifacts.component.ProjectComponentSelector;
 import org.gradle.api.artifacts.result.ComponentSelectionCause;
 import org.gradle.api.internal.artifacts.ResolvedVersionConstraint;
-import org.gradle.api.internal.artifacts.dependencies.DefaultResolvedVersionConstraint;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelector;
-import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelectorScheme;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphSelector;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.selectors.ResolvableSelectorState;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionDescriptorInternal;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionReasonInternal;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionReasons;
-import org.gradle.api.internal.attributes.ImmutableAttributesFactory;
 import org.gradle.internal.component.model.DependencyMetadata;
+import org.gradle.internal.logging.text.TreeFormatter;
 import org.gradle.internal.resolve.ModuleVersionResolveException;
 import org.gradle.internal.resolve.RejectedByAttributesVersion;
 import org.gradle.internal.resolve.RejectedByRuleVersion;
@@ -45,10 +41,9 @@ import org.gradle.internal.resolve.resolver.DependencyToComponentIdResolver;
 import org.gradle.internal.resolve.result.BuildableComponentIdResolveResult;
 import org.gradle.internal.resolve.result.ComponentIdResolveResult;
 import org.gradle.internal.resolve.result.DefaultBuildableComponentIdResolveResult;
-import org.gradle.internal.logging.text.TreeFormatter;
 
 import java.util.Collection;
-import java.util.Set;
+import java.util.List;
 
 /**
  * Resolution state for a given module version selector.
@@ -60,20 +55,14 @@ import java.util.Set;
  * In this case {@link #resolved} will be `true` and {@link ModuleResolveState#getSelected()} will point to the selected component.
  */
 class SelectorState implements DependencyGraphSelector, ResolvableSelectorState {
-    private static final Transformer<ComponentSelectionDescriptorInternal, ComponentSelectionDescriptorInternal> IDENTITY = new Transformer<ComponentSelectionDescriptorInternal, ComponentSelectionDescriptorInternal>() {
-        @Override
-        public ComponentSelectionDescriptorInternal transform(ComponentSelectionDescriptorInternal componentSelectionDescriptorInternal) {
-            return componentSelectionDescriptorInternal;
-        }
-    };
     private final Long id;
     private final DependencyState dependencyState;
     private final DependencyMetadata firstSeenDependency;
     private final DependencyToComponentIdResolver resolver;
-    private final DefaultResolvedVersionConstraint versionConstraint;
-    private final VersionSelectorScheme versionSelectorScheme;
-    private final ImmutableAttributesFactory attributesFactory;
-    private final Set<ComponentSelectionDescriptorInternal> dependencyReasons = Sets.newLinkedHashSet();
+    private final ResolvedVersionConstraint versionConstraint;
+    private final List<ComponentSelectionDescriptorInternal> dependencyReasons = Lists.newArrayListWithExpectedSize(4);
+    private final boolean isProjectSelector;
+    private final AttributeDesugaring attributeDesugaring;
 
     private ComponentIdResolveResult preferResult;
     private ComponentIdResolveResult requireResult;
@@ -92,23 +81,28 @@ class SelectorState implements DependencyGraphSelector, ResolvableSelectorState
     // evicted, but it can still be reintegrated later in a different path.
     private int outgoingEdgeCount;
 
-    SelectorState(Long id, DependencyState dependencyState, DependencyToComponentIdResolver resolver, VersionSelectorScheme versionSelectorScheme, ResolveState resolveState, ModuleIdentifier targetModuleId) {
+    SelectorState(Long id, DependencyState dependencyState, DependencyToComponentIdResolver resolver, ResolveState resolveState, ModuleIdentifier targetModuleId) {
         this.id = id;
         this.resolver = resolver;
-        this.versionSelectorScheme = versionSelectorScheme;
         this.targetModule = resolveState.getModule(targetModuleId);
-        this.attributesFactory = resolveState.getAttributesFactory();
-
         update(dependencyState);
         this.dependencyState = dependencyState;
         this.firstSeenDependency = dependencyState.getDependency();
-        this.versionConstraint = resolveVersionConstraint(firstSeenDependency.getSelector());
+        this.versionConstraint = resolveState.resolveVersionConstraint(firstSeenDependency.getSelector());
+        this.isProjectSelector = getSelector() instanceof ProjectComponentSelector;
+        this.attributeDesugaring = resolveState.getAttributeDesugaring();
     }
 
-    public void use() {
+    @Override
+    public boolean isProject() {
+        // this is cached because used very often in sorting selectors
+        return isProjectSelector;
+    }
+
+    public void use(boolean deferSelection) {
         outgoingEdgeCount++;
         if (outgoingEdgeCount == 1) {
-            targetModule.addSelector(this);
+            targetModule.addSelector(this, deferSelection);
         }
     }
 
@@ -125,13 +119,6 @@ class SelectorState implements DependencyGraphSelector, ResolvableSelectorState
         resolved = false;
     }
 
-    private DefaultResolvedVersionConstraint resolveVersionConstraint(ComponentSelector selector) {
-        if (selector instanceof ModuleComponentSelector) {
-            return new DefaultResolvedVersionConstraint(((ModuleComponentSelector) selector).getVersionConstraint(), versionSelectorScheme);
-        }
-        return null;
-    }
-
     @Override
     public Long getResultId() {
         return id;
@@ -144,7 +131,7 @@ class SelectorState implements DependencyGraphSelector, ResolvableSelectorState
 
     @Override
     public ComponentSelector getRequested() {
-        return selectorWithDesugaredAttributes(dependencyState.getRequested());
+        return attributeDesugaring.desugarSelector(dependencyState.getRequested());
     }
 
     public ModuleResolveState getTargetModule() {
@@ -324,10 +311,6 @@ class SelectorState implements DependencyGraphSelector, ResolvableSelectorState
         return fromLock;
     }
 
-    private ComponentSelector selectorWithDesugaredAttributes(ComponentSelector selector) {
-        return AttributeDesugaring.desugarSelector(selector, attributesFactory);
-    }
-
     public void update(DependencyState dependencyState) {
         if (dependencyState != this.dependencyState) {
             if (!forced && dependencyState.isForced()) {
