@@ -16,18 +16,22 @@
 
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.result;
 
+import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
+import com.google.common.collect.Maps;
 import org.gradle.api.artifacts.VersionConstraint;
 import org.gradle.api.artifacts.component.BuildIdentifier;
 import org.gradle.api.artifacts.component.ComponentSelector;
 import org.gradle.api.artifacts.component.LibraryComponentSelector;
 import org.gradle.api.artifacts.component.ModuleComponentSelector;
 import org.gradle.api.attributes.AttributeContainer;
+import org.gradle.api.capabilities.Capability;
 import org.gradle.api.internal.artifacts.DefaultModuleIdentifier;
 import org.gradle.api.internal.artifacts.ImmutableVersionConstraint;
 import org.gradle.api.internal.artifacts.dependencies.DefaultImmutableVersionConstraint;
 import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.internal.component.external.model.DefaultModuleComponentSelector;
+import org.gradle.internal.component.external.model.ImmutableCapability;
 import org.gradle.internal.component.local.model.DefaultLibraryComponentSelector;
 import org.gradle.internal.component.local.model.DefaultProjectComponentSelector;
 import org.gradle.internal.serialize.AbstractSerializer;
@@ -36,39 +40,46 @@ import org.gradle.internal.serialize.Encoder;
 import org.gradle.util.Path;
 
 import java.io.IOException;
+import java.util.Collection;
+import java.util.Collections;
 import java.util.List;
+import java.util.Map;
 
 public class ComponentSelectorSerializer extends AbstractSerializer<ComponentSelector> {
-    private final AttributeContainerSerializer attributeContainerSerializer;
+    private final OptimizingAttributeContainerSerializer attributeContainerSerializer;
     private final BuildIdentifierSerializer buildIdentifierSerializer;
 
     public ComponentSelectorSerializer(AttributeContainerSerializer attributeContainerSerializer) {
-        this.attributeContainerSerializer = attributeContainerSerializer;
+        this.attributeContainerSerializer = new OptimizingAttributeContainerSerializer(attributeContainerSerializer);
         this.buildIdentifierSerializer = new BuildIdentifierSerializer();
     }
 
+    void reset() {
+        attributeContainerSerializer.reset();
+    }
+
     public ComponentSelector read(Decoder decoder) throws IOException {
         byte id = decoder.readByte();
 
         if (Implementation.ROOT_PROJECT.getId() == id) {
             BuildIdentifier buildIdentifier = buildIdentifierSerializer.read(decoder);
             String projectName = decoder.readString();
-            return new DefaultProjectComponentSelector(buildIdentifier, Path.ROOT, Path.ROOT, projectName, readAttributes(decoder));
+            return new DefaultProjectComponentSelector(buildIdentifier, Path.ROOT, Path.ROOT, projectName, readAttributes(decoder), readCapabilities(decoder));
         } else if (Implementation.ROOT_BUILD_PROJECT.getId() == id) {
             BuildIdentifier buildIdentifier = buildIdentifierSerializer.read(decoder);
             Path projectPath = Path.path(decoder.readString());
-            return new DefaultProjectComponentSelector(buildIdentifier, projectPath, projectPath, projectPath.getName(), readAttributes(decoder));
+            return new DefaultProjectComponentSelector(buildIdentifier, projectPath, projectPath, projectPath.getName(), readAttributes(decoder), readCapabilities(decoder));
         } else if (Implementation.OTHER_BUILD_ROOT_PROJECT.getId() == id) {
             BuildIdentifier buildIdentifier = buildIdentifierSerializer.read(decoder);
             Path identityPath = Path.path(decoder.readString());
-            return new DefaultProjectComponentSelector(buildIdentifier, identityPath, Path.ROOT, identityPath.getName(), readAttributes(decoder));
+            return new DefaultProjectComponentSelector(buildIdentifier, identityPath, Path.ROOT, identityPath.getName(), readAttributes(decoder), readCapabilities(decoder));
         } else if (Implementation.OTHER_BUILD_PROJECT.getId() == id) {
             BuildIdentifier buildIdentifier = buildIdentifierSerializer.read(decoder);
             Path identityPath = Path.path(decoder.readString());
             Path projectPath = Path.path(decoder.readString());
-            return new DefaultProjectComponentSelector(buildIdentifier, identityPath, projectPath, projectPath.getName(), readAttributes(decoder));
+            return new DefaultProjectComponentSelector(buildIdentifier, identityPath, projectPath, projectPath.getName(), readAttributes(decoder), readCapabilities(decoder));
         } else if (Implementation.MODULE.getId() == id) {
-            return DefaultModuleComponentSelector.newSelector(DefaultModuleIdentifier.newId(decoder.readString(), decoder.readString()), readVersionConstraint(decoder), readAttributes(decoder));
+            return DefaultModuleComponentSelector.newSelector(DefaultModuleIdentifier.newId(decoder.readString(), decoder.readString()), readVersionConstraint(decoder), readAttributes(decoder), readCapabilities(decoder));
         } else if (Implementation.LIBRARY.getId() == id) {
             return new DefaultLibraryComponentSelector(decoder.readString(), decoder.readNullableString(), decoder.readNullableString());
         }
@@ -92,6 +103,27 @@ public class ComponentSelectorSerializer extends AbstractSerializer<ComponentSel
         return new DefaultImmutableVersionConstraint(prefers, requires, strictly, rejects);
     }
 
+    private List<Capability> readCapabilities(Decoder decoder) throws IOException {
+        int size = decoder.readSmallInt();
+        if (size == 0) {
+            return Collections.emptyList();
+        }
+        ImmutableList.Builder<Capability> builder = ImmutableList.builderWithExpectedSize(size);
+        for (int i=0; i<size; i++) {
+            builder.add(new ImmutableCapability(decoder.readString(), decoder.readString(), decoder.readNullableString()));
+        }
+        return builder.build();
+    }
+
+    private void writeCapabilities(Encoder encoder, Collection<Capability> capabilities) throws IOException {
+        encoder.writeSmallInt(capabilities.size());
+        for (Capability capability : capabilities) {
+            encoder.writeString(capability.getGroup());
+            encoder.writeString(capability.getName());
+            encoder.writeNullableString(capability.getVersion());
+        }
+    }
+
     public void write(Encoder encoder, ComponentSelector value) throws IOException {
         if (value == null) {
             throw new IllegalArgumentException("Provided component selector may not be null");
@@ -108,27 +140,32 @@ public class ComponentSelectorSerializer extends AbstractSerializer<ComponentSel
             VersionConstraint versionConstraint = moduleComponentSelector.getVersionConstraint();
             writeVersionConstraint(encoder, versionConstraint);
             writeAttributes(encoder, moduleComponentSelector.getAttributes());
+            writeCapabilities(encoder, moduleComponentSelector.getRequestedCapabilities());
         } else if (implementation == Implementation.ROOT_PROJECT) {
             DefaultProjectComponentSelector projectComponentSelector = (DefaultProjectComponentSelector) value;
             buildIdentifierSerializer.write(encoder, projectComponentSelector.getBuildIdentifier());
             encoder.writeString(projectComponentSelector.getProjectName());
             writeAttributes(encoder, projectComponentSelector.getAttributes());
+            writeCapabilities(encoder, projectComponentSelector.getRequestedCapabilities());
         } else if (implementation == Implementation.ROOT_BUILD_PROJECT) {
             DefaultProjectComponentSelector projectComponentSelector = (DefaultProjectComponentSelector) value;
             buildIdentifierSerializer.write(encoder, projectComponentSelector.getBuildIdentifier());
             encoder.writeString(projectComponentSelector.getProjectPath());
             writeAttributes(encoder, projectComponentSelector.getAttributes());
+            writeCapabilities(encoder, projectComponentSelector.getRequestedCapabilities());
         } else if (implementation == Implementation.OTHER_BUILD_ROOT_PROJECT) {
             DefaultProjectComponentSelector projectComponentSelector = (DefaultProjectComponentSelector) value;
             buildIdentifierSerializer.write(encoder, projectComponentSelector.getBuildIdentifier());
             encoder.writeString(projectComponentSelector.getIdentityPath().getPath());
             writeAttributes(encoder, projectComponentSelector.getAttributes());
+            writeCapabilities(encoder, projectComponentSelector.getRequestedCapabilities());
         } else if (implementation == Implementation.OTHER_BUILD_PROJECT) {
             DefaultProjectComponentSelector projectComponentSelector = (DefaultProjectComponentSelector) value;
             buildIdentifierSerializer.write(encoder, projectComponentSelector.getBuildIdentifier());
             encoder.writeString(projectComponentSelector.getIdentityPath().getPath());
             encoder.writeString(projectComponentSelector.getProjectPath());
             writeAttributes(encoder, projectComponentSelector.getAttributes());
+            writeCapabilities(encoder, projectComponentSelector.getRequestedCapabilities());
         } else if (implementation == Implementation.LIBRARY) {
             LibraryComponentSelector libraryComponentSelector = (LibraryComponentSelector) value;
             encoder.writeString(libraryComponentSelector.getProjectPath());
@@ -195,4 +232,65 @@ public class ComponentSelectorSerializer extends AbstractSerializer<ComponentSel
             return id;
         }
     }
+
+    private static class OptimizingAttributeContainerSerializer implements AttributeContainerSerializer {
+        // We use an identity hashmap for performance, because we know that our attributes
+        // are generated by a factory which guarantees same instances
+        private final Map<AttributeContainer, Integer> writeIndex = Maps.newIdentityHashMap();
+        private final List<ImmutableAttributes> readIndex = Lists.newArrayList();
+        private final AttributeContainerSerializer delegate;
+
+        private OptimizingAttributeContainerSerializer(AttributeContainerSerializer delegate) {
+            this.delegate = delegate;
+        }
+
+        /**
+         * If the cache expired, or that it contains too many entries,
+         * it is possible for the same file to be read multiple times, in which
+         * case the internal state of the reader _must_ be reset.
+         */
+        public void reset() {
+            writeIndex.clear();
+            readIndex.clear();
+        }
+
+        @Override
+        public ImmutableAttributes read(Decoder decoder) throws IOException {
+            boolean empty = decoder.readBoolean();
+            if (empty) {
+                return ImmutableAttributes.EMPTY;
+            }
+            int idx = decoder.readSmallInt();
+            ImmutableAttributes attributes;
+            if (idx == readIndex.size()) {
+                // new entry
+                attributes = delegate.read(decoder);
+                readIndex.add(attributes);
+            } else {
+                attributes = readIndex.get(idx);
+            }
+            return attributes;
+        }
+
+        @Override
+        public void write(Encoder encoder, AttributeContainer container) throws IOException {
+            if (container.isEmpty()) {
+                encoder.writeBoolean(true);
+                return;
+            } else {
+                encoder.writeBoolean(false);
+            }
+            Integer idx = writeIndex.get(container);
+            if (idx == null) {
+                // new value
+                encoder.writeSmallInt(writeIndex.size());
+                writeIndex.put(container, writeIndex.size());
+                delegate.write(encoder, container);
+            } else {
+                // known value, only write index
+                encoder.writeSmallInt(idx);
+            }
+        }
+    }
+
 }
