@@ -16,15 +16,15 @@
 package org.gradle.api.internal.tasks.compile;
 
 import com.google.common.base.Joiner;
+import com.google.common.base.Predicate;
+import com.google.common.collect.ImmutableSet;
+import com.google.common.collect.Iterables;
 import com.google.common.collect.Lists;
 import org.gradle.api.Transformer;
-import org.gradle.api.file.FileCollection;
-import org.gradle.api.internal.file.collections.SimpleFileCollection;
-import org.gradle.api.internal.tasks.SimpleWorkResult;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
-import org.gradle.api.specs.Spec;
 import org.gradle.api.tasks.WorkResult;
+import org.gradle.api.tasks.WorkResults;
 import org.gradle.language.base.internal.compile.Compiler;
 import org.gradle.util.CollectionUtils;
 
@@ -61,8 +61,9 @@ public class NormalizingGroovyCompiler implements Compiler<GroovyJavaJointCompil
                 return '.' + extension;
             }
         });
-        FileCollection filtered = spec.getSource().filter(new Spec<File>() {
-            public boolean isSatisfiedBy(File element) {
+        Iterable<File> filtered = Iterables.filter(spec.getSourceFiles(), new Predicate<File>() {
+            @Override
+            public boolean apply(File element) {
                 for (String fileExtension : fileExtensions) {
                     if (hasExtension(element, fileExtension)) {
                         return true;
@@ -72,7 +73,7 @@ public class NormalizingGroovyCompiler implements Compiler<GroovyJavaJointCompil
             }
         });
 
-        spec.setSource(new SimpleFileCollection(filtered.getFiles()));
+        spec.setSourceFiles(ImmutableSet.copyOf(filtered));
     }
 
     private void resolveClasspath(GroovyJavaJointCompileSpec spec) {
@@ -98,7 +99,7 @@ public class NormalizingGroovyCompiler implements Compiler<GroovyJavaJointCompil
 
         StringBuilder builder = new StringBuilder();
         builder.append("Source files to be compiled:");
-        for (File file : spec.getSource()) {
+        for (File file : spec.getSourceFiles()) {
             builder.append('\n');
             builder.append(file);
         }
@@ -127,7 +128,7 @@ public class NormalizingGroovyCompiler implements Compiler<GroovyJavaJointCompil
                 throw e;
             }
             LOGGER.debug("Ignoring compilation failure.");
-            return new SimpleWorkResult(false);
+            return WorkResults.didWork(false);
         }
     }
 }
