@@ -16,15 +16,13 @@
 
 package org.gradle.api.internal.tasks.compile.incremental.deps;
 
-import com.google.common.collect.HashMultimap;
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Multimap;
 import com.google.common.collect.Sets;
 import it.unimi.dsi.fastutil.ints.IntSet;
-import org.gradle.api.internal.tasks.compile.incremental.processing.AnnotationProcessingResult;
 
-import java.io.File;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
@@ -33,35 +31,25 @@ import java.util.Set;
 public class ClassDependentsAccumulator {
 
     private final Set<String> dependenciesToAll = Sets.newHashSet();
-    private final Set<String> aggregatedTypes = Sets.newHashSet();
-    private final Set<String> dependentsOnAll = Sets.newHashSet();
-    private final Map<String, String> filePathToClassName = new HashMap<String, String>();
     private final Map<String, Set<String>> dependents = new HashMap<String, Set<String>>();
-    private final Map<String, IntSet> classesToConstants = new HashMap<String, IntSet>();
+    private final ImmutableMap.Builder<String, IntSet> classesToConstants = ImmutableMap.builder();
     private final Set<String> seenClasses = Sets.newHashSet();
-    private final Multimap<String, String> parentToChildren = HashMultimap.create();
     private String fullRebuildCause;
 
-    public ClassDependentsAccumulator() {
-    }
-
-    public void addClass(File classFile, ClassAnalysis classAnalysis) {
-        addClass(classAnalysis);
-        filePathToClassName.put(classFile.getAbsolutePath(), classAnalysis.getClassName());
-    }
-
     public void addClass(ClassAnalysis classAnalysis) {
-        addClass(classAnalysis.getClassName(), classAnalysis.isDependencyToAll(), classAnalysis.getClassDependencies(), classAnalysis.getConstants(), classAnalysis.getSuperTypes());
+        addClass(classAnalysis.getClassName(), classAnalysis.isDependencyToAll(), classAnalysis.getClassDependencies(), classAnalysis.getConstants());
     }
 
-    public void addClass(String className, boolean dependencyToAll, Iterable<String> classDependencies, IntSet constants, Set<String> superTypes) {
+    public void addClass(String className, boolean dependencyToAll, Iterable<String> classDependencies, IntSet constants) {
         if (seenClasses.contains(className)) {
             // same classes may be found in different classpath trees/jars
             // and we keep only the first one
             return;
         }
         seenClasses.add(className);
-        classesToConstants.put(className, constants);
+        if (!constants.isEmpty()) {
+            classesToConstants.put(className, constants);
+        }
         if (dependencyToAll) {
             dependenciesToAll.add(className);
             dependents.remove(className);
@@ -71,9 +59,6 @@ public class ClassDependentsAccumulator {
                 addDependency(dependency, className);
             }
         }
-        for (String superType : superTypes) {
-            parentToChildren.put(superType, className);
-        }
     }
 
     private Set<String> rememberClass(String className) {
@@ -85,7 +70,8 @@ public class ClassDependentsAccumulator {
         return d;
     }
 
-    public Map<String, DependentsSet> getDependentsMap() {
+    @VisibleForTesting
+    Map<String, DependentsSet> getDependentsMap() {
         if (dependenciesToAll.isEmpty() && dependents.isEmpty()) {
             return Collections.emptyMap();
         }
@@ -94,28 +80,14 @@ public class ClassDependentsAccumulator {
             builder.put(s, DependentsSet.dependencyToAll());
         }
         for (Map.Entry<String, Set<String>> entry : dependents.entrySet()) {
-            builder.put(entry.getKey(), DependentsSet.dependents(entry.getValue()));
+            builder.put(entry.getKey(), DependentsSet.dependentClasses(entry.getValue()));
         }
         return builder.build();
     }
 
-    public Map<String, IntSet> getClassesToConstants() {
-        return classesToConstants;
-    }
-
-    public void addAnnotationProcessingResult(AnnotationProcessingResult annotationProcessingResult) {
-        for (Map.Entry<String, Set<String>> entry : annotationProcessingResult.getGeneratedTypesWithIsolatedOrigin().entrySet()) {
-            String originatingType = entry.getKey();
-            for (String generatedType : entry.getValue()) {
-                addDependency(originatingType, generatedType);
-                addDependency(generatedType, originatingType);
-            }
-        }
-        aggregatedTypes.addAll(annotationProcessingResult.getAggregatedTypes());
-        dependentsOnAll.addAll(annotationProcessingResult.getGeneratedAggregatingTypes());
-        if (annotationProcessingResult.getFullRebuildCause() != null) {
-            fullRebuildCause = annotationProcessingResult.getFullRebuildCause();
-        }
+    @VisibleForTesting
+    Map<String, IntSet> getClassesToConstants() {
+        return classesToConstants.build();
     }
 
     private void addDependency(String dependency, String dependent) {
@@ -128,7 +100,7 @@ public class ClassDependentsAccumulator {
     }
 
     public ClassSetAnalysisData getAnalysis() {
-        return new ClassSetAnalysisData(filePathToClassName, getDependentsMap(), getClassesToConstants(), asMap(parentToChildren), DependentsSet.dependents(aggregatedTypes), DependentsSet.dependents(dependentsOnAll), fullRebuildCause);
+        return new ClassSetAnalysisData(ImmutableSet.copyOf(seenClasses), getDependentsMap(), getClassesToConstants(), fullRebuildCause);
     }
 
     private static <K, V> Map<K, Set<V>> asMap(Multimap<K, V> multimap) {
