@@ -16,14 +16,13 @@
 
 package org.gradle.internal.logging.sink;
 
-import net.jcip.annotations.ThreadSafe;
+import javax.annotation.concurrent.ThreadSafe;
 import org.gradle.api.logging.LogLevel;
 import org.gradle.api.logging.StandardOutputListener;
 import org.gradle.api.logging.configuration.ConsoleOutput;
 import org.gradle.internal.Factory;
 import org.gradle.internal.event.ListenerBroadcast;
 import org.gradle.internal.logging.config.LoggingRouter;
-import org.gradle.internal.logging.console.AnsiConsole;
 import org.gradle.internal.logging.console.BuildLogLevelFilterRenderer;
 import org.gradle.internal.logging.console.BuildStatusRenderer;
 import org.gradle.internal.logging.console.ColorMap;
@@ -55,7 +54,6 @@ import org.gradle.internal.time.Clock;
 
 import javax.annotation.Nullable;
 import java.io.OutputStream;
-import java.io.OutputStreamWriter;
 import java.util.concurrent.atomic.AtomicReference;
 
 /**
@@ -159,25 +157,9 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     public void attachConsole(OutputStream outputStream, OutputStream errorStream, ConsoleOutput consoleOutput, @Nullable ConsoleMetaData consoleMetadata) {
         synchronized (lock) {
             if (consoleMetadata == null) {
-                consoleMetadata = consoleOutput == ConsoleOutput.Plain ? FallbackConsoleMetaData.NOT_ATTACHED : FallbackConsoleMetaData.ATTACHED;
-            }
-            StandardOutputListener outputListener = new StreamBackedStandardOutputListener(outputStream);
-            StandardOutputListener errorListener = new StreamBackedStandardOutputListener(errorStream);
-            if (consoleOutput == ConsoleOutput.Plain) {
-                addPlainConsole(outputListener, errorListener, consoleMetadata.isStdOut() && consoleMetadata.isStdErr());
-            } else {
-                Console console;
-                if (consoleMetadata.isStdOut()) {
-                    OutputStreamWriter writer = new OutputStreamWriter(outputStream);
-                    console = new AnsiConsole(writer, writer, getColourMap(), consoleMetadata, true);
-                } else if (consoleMetadata.isStdErr()) {
-                    OutputStreamWriter writer = new OutputStreamWriter(errorStream);
-                    console = new AnsiConsole(writer, writer, getColourMap(), consoleMetadata, true);
-                } else {
-                    console = null;
-                }
-                addRichConsole(console, outputListener, errorListener, consoleMetadata, consoleOutput == ConsoleOutput.Verbose);
+                consoleMetadata = FallbackConsoleMetaData.NOT_ATTACHED;
             }
+            ConsoleConfigureAction.execute(this, consoleOutput, consoleMetadata, outputStream, errorStream);
         }
     }
 
@@ -248,37 +230,49 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
         }
     }
 
-    public OutputEventRenderer addRichConsole(Console console, ConsoleMetaData consoleMetaData) {
-        return addRichConsole(console, consoleMetaData, false);
+    public void addRichConsoleWithErrorOutputOnStdout(Console stdout, ConsoleMetaData consoleMetaData, boolean verbose) {
+        OutputEventListener consoleListener = new StyledTextOutputBackedRenderer(stdout.getBuildOutputArea());
+        OutputEventListener consoleChain = getConsoleChainWithDynamicStdout(stdout, consoleMetaData, verbose, consoleListener);
+        addConsoleChain(consoleChain);
     }
 
-    public OutputEventRenderer addRichConsole(Console console, ConsoleMetaData consoleMetaData, boolean verbose) {
-        return addRichConsole(console, new StreamBackedStandardOutputListener((Appendable) originalStdOut), new StreamBackedStandardOutputListener((Appendable) originalStdErr), consoleMetaData, verbose);
+    public void addRichConsole(Console stdout, Console stderr, ConsoleMetaData consoleMetaData, boolean verbose) {
+        OutputEventListener stdoutChain = new StyledTextOutputBackedRenderer(stdout.getBuildOutputArea());
+        OutputEventListener stderrChain = new FlushConsoleListener(stderr, new StyledTextOutputBackedRenderer(stderr.getBuildOutputArea()));
+        OutputEventListener consoleListener = new ErrorOutputDispatchingListener(stderrChain, stdoutChain);
+        OutputEventListener consoleChain = getConsoleChainWithDynamicStdout(stdout, consoleMetaData, verbose, consoleListener);
+        addConsoleChain(consoleChain);
     }
 
-    private OutputEventRenderer addRichConsole(Console console, StandardOutputListener outputListener, StandardOutputListener errorListener, ConsoleMetaData consoleMetaData, boolean verbose) {
-        OutputEventListener consoleChain;
-        boolean stdoutAttachedToConsole = consoleMetaData.isStdOut();
-        boolean stderrAttachedToConsole = consoleMetaData.isStdErr();
-        if (stdoutAttachedToConsole && stderrAttachedToConsole) {
-            OutputEventListener consoleListener = new StyledTextOutputBackedRenderer(console.getBuildOutputArea());
-            consoleChain = getRichConsoleChain(console, consoleMetaData, verbose, consoleListener);
-        } else if (stdoutAttachedToConsole) {
-            OutputEventListener stderrChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(errorListener));
-            OutputEventListener consoleListener = new ErrorOutputDispatchingListener(stderrChain, new StyledTextOutputBackedRenderer(console.getBuildOutputArea()), false);
-            consoleChain = getRichConsoleChain(console, consoleMetaData, verbose, consoleListener);
-        } else if (stderrAttachedToConsole) {
-            OutputEventListener stdoutChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(outputListener));
-            OutputEventListener stderrChain = new FlushConsoleListener(console, new StyledTextOutputBackedRenderer(console.getBuildOutputArea()));
-            consoleChain = getPlainConsoleChain(stdoutChain, stderrChain, false, verbose);
-        } else {
-            consoleChain = getPlainConsoleChain(outputListener, errorListener, false, verbose);
-        }
-
-        return addConsoleChain(consoleChain);
+    public void addRichConsole(Console stdout, OutputStream stderr, ConsoleMetaData consoleMetaData, boolean verbose) {
+        OutputEventListener stdoutChain = new StyledTextOutputBackedRenderer(stdout.getBuildOutputArea());
+        OutputEventListener stderrChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(new StreamBackedStandardOutputListener(stderr)));
+        OutputEventListener consoleListener = new ErrorOutputDispatchingListener(stderrChain, stdoutChain);
+        OutputEventListener consoleChain = getConsoleChainWithDynamicStdout(stdout, consoleMetaData, verbose, consoleListener);
+        addConsoleChain(consoleChain);
     }
 
-    private OutputEventListener getRichConsoleChain(Console console, ConsoleMetaData consoleMetaData, boolean verbose, OutputEventListener consoleListener) {
+    public void addRichConsole(OutputStream stdout, Console stderr, boolean verbose) {
+        OutputEventListener stdoutChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(new StreamBackedStandardOutputListener(stdout)));
+        OutputEventListener stderrChain = new FlushConsoleListener(stderr, new StyledTextOutputBackedRenderer(stderr.getBuildOutputArea()));
+        OutputEventListener consoleListener = new ErrorOutputDispatchingListener(stderrChain, stdoutChain);
+        OutputEventListener consoleChain = getConsoleChainWithoutDynamicStdout(consoleListener, verbose);
+        addConsoleChain(consoleChain);
+    }
+
+    public void addPlainConsoleWithErrorOutputOnStdout(OutputStream stdout) {
+        OutputEventListener stdoutChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(new StreamBackedStandardOutputListener(stdout)));
+        addConsoleChain(getConsoleChainWithoutDynamicStdout(stdoutChain, true));
+    }
+
+    public void addPlainConsole(OutputStream stdout, OutputStream stderr) {
+        OutputEventListener stdoutChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(new StreamBackedStandardOutputListener(stdout)));
+        OutputEventListener stderrChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(new StreamBackedStandardOutputListener(stderr)));
+        OutputEventListener outputListener = new ErrorOutputDispatchingListener(stderrChain, stdoutChain);
+        addConsoleChain(getConsoleChainWithoutDynamicStdout(outputListener, true));
+    }
+
+    private OutputEventListener getConsoleChainWithDynamicStdout(Console console, ConsoleMetaData consoleMetaData, boolean verbose, OutputEventListener consoleListener) {
         return throttled(
             new UserInputConsoleRenderer(
                 new BuildStatusRenderer(
@@ -290,35 +284,22 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
                         new DefaultWorkInProgressFormatter(consoleMetaData),
                         new ConsoleLayoutCalculator(consoleMetaData)
                     ),
-                console.getStatusBar(), console, consoleMetaData),
-            console)
+                    console.getStatusBar(), console, consoleMetaData),
+                console)
         );
     }
 
-    public OutputEventRenderer addPlainConsole(boolean redirectStderr) {
-        return addPlainConsole(new StreamBackedStandardOutputListener((Appendable) originalStdOut), new StreamBackedStandardOutputListener((Appendable) originalStdErr), redirectStderr);
-    }
-
-    private OutputEventRenderer addPlainConsole(StandardOutputListener outputListener, StandardOutputListener errorListener, boolean redirectStderr) {
-        return addConsoleChain(getPlainConsoleChain(outputListener, errorListener, redirectStderr, true));
-    }
-
-    private OutputEventListener getPlainConsoleChain(StandardOutputListener outputListener, StandardOutputListener errorListener, boolean redirectStderr, boolean verbose) {
-        final OutputEventListener stdoutChain = new UserInputStandardOutputRenderer(new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(outputListener)));
-        final OutputEventListener stderrChain = new StyledTextOutputBackedRenderer(new StreamingStyledTextOutput(errorListener));
-        return getPlainConsoleChain(stdoutChain, stderrChain, redirectStderr, verbose);
-    }
-
-    private OutputEventListener getPlainConsoleChain(OutputEventListener stdoutChain, OutputEventListener stderrChain, boolean redirectStderr, boolean verbose) {
+    private OutputEventListener getConsoleChainWithoutDynamicStdout(OutputEventListener outputListener, boolean verbose) {
         return throttled(
-            new BuildLogLevelFilterRenderer(
-                new GroupingProgressLogEventGenerator(
-                    new ErrorOutputDispatchingListener(stderrChain, stdoutChain, redirectStderr),
-                    new PrettyPrefixedLogHeaderFormatter(),
-                    verbose
+            new UserInputStandardOutputRenderer(
+                new BuildLogLevelFilterRenderer(
+                    new GroupingProgressLogEventGenerator(
+                        outputListener,
+                        new PrettyPrefixedLogHeaderFormatter(),
+                        verbose
+                    )
                 )
-            )
-        );
+        ));
     }
 
     private OutputEventListener throttled(OutputEventListener consoleChain) {
