@@ -43,8 +43,6 @@ public class BaseMavenDeployer extends AbstractMavenResolver implements MavenDep
     // todo remove this property once configuration can handle normal file system dependencies
     private List<File> protocolProviderJars = new ArrayList<File>();
 
-    private boolean uniqueVersion = true;
-
     public BaseMavenDeployer(PomFilterContainer pomFilterContainer, ArtifactPomContainer artifactPomContainer, LoggingManagerInternal loggingManager,
                              MavenSettingsProvider mavenSettingsProvider, LocalMavenRepositoryLocator mavenRepositoryLocator, ObjectFactory objectFactory) {
         super(pomFilterContainer, artifactPomContainer, loggingManager, mavenSettingsProvider, mavenRepositoryLocator, objectFactory);
@@ -53,8 +51,6 @@ public class BaseMavenDeployer extends AbstractMavenResolver implements MavenDep
     protected MavenPublishAction createPublishAction(String packaging, MavenProjectIdentity projectIdentity, LocalMavenRepositoryLocator mavenRepositoryLocator) {
         MavenWagonDeployAction deployAction = new MavenWagonDeployAction(packaging, projectIdentity, getJars());
         deployAction.setLocalMavenRepositoryLocation(mavenRepositoryLocator.getLocalMavenRepository());
-        deployAction.produceLegacyMavenMetadata();
-        deployAction.setUniqueVersion(isUniqueVersion());
         deployAction.setRepositories(remoteRepository, remoteSnapshotRepository);
         return deployAction;
     }
@@ -90,12 +86,4 @@ public class BaseMavenDeployer extends AbstractMavenResolver implements MavenDep
     public void setConfiguration(Configuration configuration) {
         this.configuration = configuration;
     }
-
-    public boolean isUniqueVersion() {
-        return uniqueVersion;
-    }
-
-    public void setUniqueVersion(boolean uniqueVersion) {
-        this.uniqueVersion = uniqueVersion;
-    }
 }
