@@ -17,8 +17,17 @@
 package org.gradle.jvm.internal;
 
 import org.gradle.jvm.JarBinarySpec;
+import org.gradle.platform.base.DependencySpec;
 import org.gradle.platform.base.internal.BinarySpecInternal;
-import org.gradle.platform.base.internal.ComponentSpecAware;
 
-public interface JarBinarySpecInternal extends JarBinarySpec, BinarySpecInternal, ComponentSpecAware {
+import java.util.Collection;
+
+public interface JarBinarySpecInternal extends JarBinarySpec, BinarySpecInternal, JvmBinarySpecInternal, WithJvmAssembly, WithDependencies {
+    void setApiDependencies(Collection<DependencySpec> apiDependencies);
+
+    Collection<DependencySpec> getApiDependencies();
+
+    JarFile getApiJar();
+
+    JarFile getRuntimeJar();
 }
