@@ -16,15 +16,14 @@
 
 package org.gradle.internal.fingerprint.impl;
 
-import org.gradle.api.internal.cache.StringInterner;
 import org.gradle.api.tasks.FileNormalizer;
 import org.gradle.internal.fingerprint.OutputNormalizer;
 import org.gradle.internal.snapshot.FileSystemSnapshotter;
 
-public class OutputFileCollectionFingerprinter extends AbstractPathOnlyFileCollectionFingerprinter {
+public class OutputFileCollectionFingerprinter extends AbstractFileCollectionFingerprinter {
 
-    public OutputFileCollectionFingerprinter(StringInterner stringInterner, FileSystemSnapshotter fileSystemSnapshotter) {
-        super(AbsolutePathFingerprintingStrategy.IGNORE_MISSING, stringInterner, fileSystemSnapshotter);
+    public OutputFileCollectionFingerprinter(FileSystemSnapshotter fileSystemSnapshotter) {
+        super(AbsolutePathFingerprintingStrategy.IGNORE_MISSING, fileSystemSnapshotter);
     }
 
     @Override
