@@ -19,8 +19,10 @@ import org.gradle.api.GradleException;
 import org.gradle.api.Incubating;
 import org.gradle.api.tasks.AbstractExecTask;
 import org.gradle.api.tasks.Input;
+import org.gradle.api.tasks.Internal;
 import org.gradle.api.tasks.OutputDirectory;
 import org.gradle.api.tasks.TaskAction;
+import org.gradle.api.tasks.VerificationTask;
 import org.gradle.internal.logging.ConsoleRenderer;
 
 import java.io.File;
@@ -29,7 +31,7 @@ import java.io.File;
  * Runs a compiled and installed test executable.
  */
 @Incubating
-public class RunTestExecutable extends AbstractExecTask<RunTestExecutable> {
+public class RunTestExecutable extends AbstractExecTask<RunTestExecutable> implements VerificationTask {
     /**
      * The directory where the results should be generated.
      */
@@ -80,15 +82,22 @@ public class RunTestExecutable extends AbstractExecTask<RunTestExecutable> {
         this.outputDir = outputDir;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     @Input
     public boolean getIgnoreFailures() {
         return ignoreFailures;
     }
 
+    @Internal
     public boolean isIgnoreFailures() {
         return ignoreFailures;
     }
 
+    /**
+     * {@inheritDoc}
+     */
     public void setIgnoreFailures(boolean ignoreFailures) {
         this.ignoreFailures = ignoreFailures;
     }
